@@ -272,6 +272,7 @@ shinyServer(function(input, output, session) {
       return(0)
     }
   })
+
   # Calculates best model using cNORM
   bestModel <- eventReactive(input$CalcBestModel, {
 
@@ -309,6 +310,7 @@ shinyServer(function(input, output, session) {
 
   })
 
+
   modelDensity <- reactive({
 
     MIN_RAW <- bestModel()$minRaw
@@ -713,6 +715,15 @@ shinyServer(function(input, output, session) {
     }
   )
 
+  output$exportData <- downloadHandler(
+    filename = function() {
+      paste("model.RData", sep = "")
+    },
+    content = function(file) {
+      save(preparedData(), filename = file)
+    }
+  )
+
   # Generates and plots modelled against actual data values
   output$PlotValues <- renderPlot({
 
